#!/usr/bin/env python

# Copyright 2007, Michael J. Harms
# This program is distributed under General Public License v. 3.  See the file
# COPYING for a copy of the license.
#
# Modified by Andrew Avila, 2011
# Removed dependancy on cmdline helper and used argparse instead
# changed to automatically split all chains
__description__ = \
"""
pdb_subset.py

Takes a subset of residues/chains from a pdb file.  All non-atom entries are 
left intact.
"""

__author__ = "Michael J. Harms"
__date__ = "080121"



class PdbSubsetError(Exception):
    """
    General error class for this module.
    """

    pass

def pdbSubset(pdb,chain,residues,
              atom_list=["ATOM  ","ANISOU","HETATM","TER   "]):
    """
    Grabs a subset of a pdb file, spitting out the pdb, the chain, and the 
    actual min and max in the pdb file.
    """

    out = []
    for line in pdb:

        # only look at records indicated by atom_list
        if line[0:6] not in atom_list:
            out.append(line)
            continue    
        
        # Grab only residues belonging to chain
        if chain == "all" or line[21:22] == chain:

            if line[0:6] == "HETATM":
                if line[17:20] != "HOH":
                    continue
   
            # Grab only residues above minimum and below maximum 
            if residues[0] == 0 and residues[1] == 0:
                out.append(line)
            elif int(line[22:26]) >= residues[0] and \
                        int(line[22:26]) <= residues[1]:
                out.append(line)
            else:
                continue
        else:
            continue

    if len([l for l in out if l[0:6] in atom_list]) == 0:
        err = "No residues in pdb meet subset criteria!"
        raise PdbSubsetError(err)

    # Determine the actual min and max in the pdb file
    residues = [0,0]
    all_residues = [int(l[22:26]) for l in out if l[0:6] in atom_list]
    residues[0] = min(all_residues)
    residues[1] = max(all_residues)                                 

    return out, chain, residues


def main():
    """
    If called from command line.
    """

    import argparse

    # Parse command line
    parser = argparse.ArgumentParser(description = __description__)
    #parser.add_argument('chain', type=str)
    parser.add_argument('file_list', nargs="+", type=str)
    parser.add_argument('--residues', nargs="+", type=int, default=[0, 0])
    
    args = parser.parse_args()
    #chain = args.chain
    file_list = args.file_list
    residues = args.residues
    
    for pdb_file in file_list:
        chain_list=[]
        
        # Attempt to strip .pdb extension
        try:
            pdb_id = pdb_file[:pdb_file.index(".pdb")]
        except IndexError:
            pdb_id = pdb_file

        # Read file
        f = open(pdb_file,'r')
        pdb = f.readlines()
        f.close()

        for line in pdb:
            atom_list=["ATOM  ","ANISOU","HETATM","TER   "]
            if line[0:6] in atom_list:
                if line[21:22] not in chain_list:
                    chain_list.append(line[21:22])

        # Pop out subset
        for chain in chain_list:
            pdb_out, chain, residues = pdbSubset(pdb,chain,residues)

            # Write to file
            out_file = "%s_%s.pdb" % (pdb_id,chain)
            g = open(out_file,'w')
            g.writelines(pdb_out)
            g.close()



if __name__ == "__main__":
    main()

